import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:theme_test/theme_store.dart';

void main() => runApp(const MyApp());

class MyApp extends StatefulWidget {
  const MyApp({Key? key}) : super(key: key);
  @override
  State<StatefulWidget> createState() => MyAppState();
}

class MyAppState extends State<MyApp> {
  final ThemeProvider _themeProvider = ThemeProvider();

  @override
  void initState() {
    // 初始化ThemeStore后，获取持久化存储的主题模式传到themeProvider里
    ThemeStore.init().then((value) => _themeProvider.setThemeMode(ThemeStore.getThemeMode()));
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return ChangeNotifierProvider(
      create: (context) => _themeProvider,
      child: Consumer(builder: (context, ThemeProvider provider, child) {
        return MaterialApp(
          title: 'ThemeMode Test',
          theme: ThemeData(colorScheme: const ColorScheme.light()),
          darkTheme: ThemeData(colorScheme: const ColorScheme.dark()),
          themeMode: provider.themeMode,
          home: Scaffold(
            appBar: AppBar(
              title: const Text("AppBar"),
            ),
            body: Padding(
              padding: const EdgeInsets.all(16.0),
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  Text("当前主题模式：${provider.themeMode}"),
                  Builder(
                    builder: (BuildContext context) {
                      // 当前是否暗色模式
                      bool isDarkMode = Theme.of(context).colorScheme.brightness == Brightness.dark;
                      return IconButton(
                        icon: Icon(isDarkMode ? Icons.light_mode : Icons.mode_night),
                        tooltip: "切换为${isDarkMode ? '亮色' : '暗黑'}模式",
                        onPressed: () async {
                          // 如果APP主题正处于跟随系统设置的情况下
                          if (_themeProvider.themeMode == ThemeMode.system) {
                            bool isCancel = await dialogTheme(context); // 用户是否取消了变更主题的请求
                            if (isCancel) return; // 取消则结束函数，不变更主题模式
                          }
                          // 切换主题模式
                          _themeProvider.setThemeMode(isDarkMode ? ThemeMode.light : ThemeMode.dark);
                        },
                      );
                    },
                  ),
                  Builder(builder: (context) {
                    return Row(
                      children: [
                        const Text("夜间模式："),
                        Switch(
                          value: Theme.of(context).colorScheme.brightness == Brightness.dark,
                          onChanged: (bool value) async {
                            // 如果APP主题正处于跟随系统设置的情况下
                            if (_themeProvider.themeMode == ThemeMode.system) {
                              bool isCancel = await dialogTheme(context); // 用户是否取消了变更主题的请求
                              if (isCancel) return; // 取消则结束函数，不变更主题模式
                            }
                            provider
                                .setThemeMode(provider.themeMode == ThemeMode.light ? ThemeMode.dark : ThemeMode.light);
                          },
                        ),
                      ],
                    );
                  }),
                  Builder(builder: (context) {
                    return Column(
                      children: [
                        _buildSelectCard("浅色模式", ThemeMode.light, provider),
                        _buildSelectCard("深色模式", ThemeMode.dark, provider),
                        _buildSelectCard("跟随系统", ThemeMode.system, provider),
                      ],
                    );
                  }),
                ],
              ),
            ),
          ),
        );
      }),
    );
  }

  /// 构建选择卡片
  Widget _buildSelectCard(String text, ThemeMode mode, ThemeProvider provider) {
    return Card(
      margin: EdgeInsets.zero,
      color: provider.themeMode == mode ? Colors.grey.withOpacity(0.25) : Colors.transparent,
      elevation: 0,
      shape: const RoundedRectangleBorder(borderRadius: BorderRadius.all(Radius.circular(10.0))),
      clipBehavior: Clip.antiAlias,
      child: InkWell(
        onTap: () => _themeProvider.setThemeMode(mode),
        child: Container(
          width: double.infinity,
          padding: const EdgeInsets.all(12),
          child: Text(
            text,
            style: provider.themeMode == mode ? TextStyle(color: Theme.of(context).primaryColor) : const TextStyle(),
          ),
        ),
      ),
    );
  }

  /// 弹窗提示切换主题会关闭跟随系统
  Future<bool> dialogTheme(BuildContext context) async {
    bool isCancel = true; // 是否点击了取消
    await showDialog(
      context: context,
      builder: (context) {
        return AlertDialog(
          title: const Text("提示"),
          content: const Text("手动切换主题会关闭主题跟随系统。"),
          actions: <Widget>[
            TextButton(child: const Text("取消"), onPressed: () => Navigator.pop(context)),
            TextButton(
              child: const Text("切换"),
              onPressed: () {
                isCancel = false;
                Navigator.pop(context);
              },
            ),
          ],
        );
      },
    );
    return isCancel;
  }
}

class ThemeProvider with ChangeNotifier {
  ThemeMode themeMode = ThemeMode.system;

  setThemeMode(ThemeMode mode) {
    themeMode = mode;
    notifyListeners();
  }
}
